# -*- coding: cp949 -*-
import unittest
import mockobject
import CharonBase

class CharonHTTPRequestHandlerTest(unittest.TestCase):
    handler = None
    request = None
    server = None

    def setUp(self):
        self.request = CharonBase.Request()
        self.request.connection = MockConnection()
        self.server = MockCharon()
        self.handler = CharonBase.CharonHTTPRequestHandler(self.request,self.server)
   
    def testhandleOne(self):
        self.handler.rfile.buffer.put_nowait('GET / HTTP/1.1\n')
        self.handler.rfile.buffer.put_nowait('Connection: Keep-Alive\n')
        self.handler.rfile.buffer.put_nowait('\n')
        self.handler.handleOne()
        self.assertEquals(self.request.command, 'GET')
        self.assertEquals(self.request.path, '/')
        self.assertEquals(self.request.version, 'HTTP/1.1')
        self.assertEquals(self.handler.closeConnection, 0)

    def testdoGET(self):
        import CharonKernel
        self.handler.request.path = '/'
        CharonBase.kernel = CharonKernel.CharonKernel()
        
        self.handler.doGET()
        self.assertEquals(1,1)
        

class CharonTest(unittest.TestCase):
    ch = None

    def setUp(self):
        self.ch = CharonBase.CharonHTTPServer()

    def test__init__(self):
        self.ch = CharonBase.CharonHTTPServer(a=1)
        self.assertEquals(self.ch.a, 1)

    def test_open(self):
        self.ch.socket = MockSocket()
        self.ch.allow_reuse_address = True
        self.ch.server_address = ('',8000)
        self.ch.socket.expectedCountersetsockopt.setExpected(1)
        self.ch.socket.expectedCounterbind.setExpected(1)
        self.ch.socket.expectedCounterlisten.setExpected(1)

        self.ch._open()
        self.ch.socket._verify()

    def test_close(self):
        self.ch.socket = MockSocket()
        self.ch.socket.expectedCounterclose.setExpected(1)

        self.ch._close()
        self.ch.socket._verify()

    def testprocess_request(self):
        mch = MockCharon()
        mch.socket = MockSocket()
        mch.socket.expectedCounteraccept.setExpected(1)
        mch.expectedCounterhandle.setExpected(1)
        mch.expectedCounterhandleerror.setExpected(0)
        mcn = MockConnection()
        mch.socket.acceptReturnPreset = (mcn, ('',8000))

        CharonBase.CharonHTTPServer.process_request(mch)
        mch.socket._verify()
        self.assertEquals(mcn,mch.request_recieved.connection)
        self.assertEquals(('',8000),mch.request_recieved.client_address)
        mch._verify()

    def testprocess_request_serverbusy(self):
        mch = MockCharon()
        mch.numMaxChildThreads = 0 #server is always busy
        mch.socket = MockSocket()
        mch.socket.expectedCounteraccept.setExpected(1)
        mch.expectedCounterhandle.setExpected(0)
        mch.expectedCounterhandleerror.setExpected(1)
        mch.expectedErrorCode.addExpected(503)

        CharonBase.CharonHTTPServer.process_request(mch)
        mch.socket._verify()
        mch._verify()
        
class RequestTest(unittest.TestCase):
    rq = None
    def setUp(self):
        self.rq = CharonBase.Request()
    def testclose(self):
        self.rq.connection = MockConnection()
        self.rq.connection.expectedCounterclose.setExpected(1)

        self.rq.close()
        self.rq.connection._verify()
    def testflush(self):
        self.rq.a = 1
        self.rq.connection = 1
        self.rq.flush()
        try:
            a = self.rq.a
        except:
            self.assertEquals(1,1)
        else:
            self.assertEquals(1,self.rq.connection)


class ResponseTest(unittest.TestCase):
    def setUp(self):
        class MockHandler:
            def __init__(self):
                self.wfile = MockFile()

        mh = MockHandler()
        self.response = CharonBase.Response(0,mh)
    def testsend(self):
        f = MockFile()
        self.response.code = 200
        f.buffer.put_nowait('test string 1\n')
        f.buffer.put_nowait('test string 2\n')
        f.buffer.put_nowait('test string 3\n')
        self.response.rfile = f
        self.response.send()
        self.assertEquals(1,1)
        
    

class MockSocket(mockobject.MockObject):
    def __init__ (self):
        self.expectedCountersetsockopt = mockobject.ExpectationCounter("MockSocket.setsockopt")
        self.expectedCounterbind = mockobject.ExpectationCounter("MockSocket.bind")
        self.expectedCounterlisten = mockobject.ExpectationCounter("MockSocket.listen")
        self.expectedCounterclose = mockobject.ExpectationCounter("MockSocket.close")
        self.expectedCounteraccept = mockobject.ExpectationCounter("MockSocket.accept")
        self.acceptReturnPreset = (MockConnection(), ('',8000))
    def setsockopt(self,param1,param2,value):
        self.expectedCountersetsockopt.inc()
    def bind(self,addr):
        self.expectedCounterbind.inc()
    def listen(self,queuesize):
        self.expectedCounterlisten.inc()
    def close(self):
        self.expectedCounterclose.inc()
    def accept(self):
        self.expectedCounteraccept.inc()
        return self.acceptReturnPreset

class MockConnection(mockobject.MockObject):
    def __init__(self):
        self.expectedCounterclose = mockobject.ExpectationCounter("MockConnection.close")
    def makefile(self,mode,bufsize):
        return MockFile()
    def close(self):
        self.expectedCounterclose.inc()

class MockCharon(mockobject.MockObject, CharonBase.CharonHTTPServer):
    request_recieved = None
    #Ƽ ׽Ʈ  Mock
    def __init__(self):
        CharonBase.CharonHTTPServer.__init__(self)
        self.expectedCounterhandle = mockobject.ExpectationCounter("MockCharon.handle_request")
        self.expectedCounterhandleerror = mockobject.ExpectationCounter("MockCharon.handle_error")
        self.expectedErrorCode = mockobject.ExpectationList("MockCharon.Errorcode")
    def handle(self,request):
        self.expectedCounterhandle.inc()
        self.request_recieved = request
    def handleError(self,request,errcode):
        self.expectedCounterhandleerror.inc()
        self.expectedErrorCode.addActual(errcode)

class MockFile(mockobject.MockObject):
    import Queue
    buffer = Queue.Queue() # get_nowait(), put_nowait() ̿.

    def __init__(self):
        self.bytesWritten = 0

    def closed(self):
        return False

    def close(self):
        pass

    def flush(self):
        pass

    def write(self,buf):
        print buf,
        self.bytesWritten += len(buf)
        

    def read(self,length):
        line = ''
        while 1:
            try:
                line += self.buffer.get_nowait()
            except:
                break
        return line

    def readline(self):
        try:
            line = self.buffer.get_nowait()
        except:
            line = ''
        return line
    
        
if __name__=="__main__":
    unittest.main(argv=('','-v'))
