import unittest
import mockobject
import StarSystemManager

class StarSystemTest(unittest.TestCase):
    s = None
    def setUp(self):
        self.s = StarSystemManager.StarSystem()
        self.s.a = 1
        self.s.b = 2
        self.s.c = 3
        self.s.genericRecords = ['a','b','c']
    def testStore_insert(self):
        self.s.db = MockDB()
        self.s.db.num_returnedRecords = 0
        self.s.db._addExpectedQuery("select id from ingameobject where id=0")
        self.s.db._addExpectedQuery("insert into ingameobject (a, c, b) values (1, 3, 2)")
        self.s.db._addExpectedQuery("select max(id) from ingameobject")
        self.s.store()
        self.s.db._verify()
        
    def testStore_update(self):
        self.s.db = MockDB()
        self.s.db.num_returnedRecords = 1
        self.s.db._addExpectedQuery("select id from ingameobject where id=0")
        self.s.db._addExpectedQuery("update ingameobject set a = '1', c = '3', b = '2' where id='0'")
        self.s.store()
        self.s.db._verify()
        
    def testToDict(self):
        self.assertEquals({'a': 1, 'c': 3, 'b': 2}, self.s._toDict())

    def testStore_insertRealDBRuntime(self):
        import MySQLdb
        self.s.db = MySQLdb.connect(host='203.253.23.195',db='test',user='opengp',passwd='venomdragon')
        c = self.s.db.cursor()
        c.execute('create table ingameobject (id int primary key auto_increment, a int, b int, c int)')
        self.s.store()
        c.execute('select * from ingameobject')
        actual = c.fetchall()
        c.execute('drop table ingameobject')
        self.assertEquals(actual,((1L, 1L, 2L, 3L),))
        

    def testStore_updateRealDBRuntime(self):
        import MySQLdb
        self.s.db = MySQLdb.connect(host='203.253.23.195',db='test',user='opengp',passwd='venomdragon')
        c = self.s.db.cursor()
        c.execute('create table ingameobject (id int primary key auto_increment, a int, b int, c int)')
        self.s.store()
        #self.s.id = 1 
        self.s.a = 4
        self.s.b = 5
        self.s.c = 6
        self.s.store()
        c.execute('select * from ingameobject')
        actual = c.fetchall()
        c.execute('drop table ingameobject')
        self.assertEquals(actual,((1L, 4L, 5L, 6L),))
        

    #def testAddNewStarSystem(self):
    #    self.s.db = MockDB()
    #    self.s.db.execute("insert")
    #    self.s.db._addExpectedExecutedResult("ok")
    #    db._verify()
        #self.assertEquals(True, isinstance(self.s.db.cursor(),MockCursor))

class MockDB(mockobject.MockObject):
    num_returnedRecords = 0
    returnedRecords = ((0,),)
    def __init__(self):
        #self._expectedResult = mockobject.ExpectationList("MockDB.addCalls")
        self._expectedQuery = mockobject.ExpectationList("MockDB.Query")
    def _addExpectedQuery(self, exp):
        self._expectedQuery.addExpected(exp)

    def _addExpectedExecutedResult(self, aStarSystem):
        #self._addCalls.setExpected(aStarSystem)
        pass
    def cursor(self):
        return MockCursor(self)

class MockCursor(mockobject.MockObject):
    parent = None
    def __init__(self,parent):
        self.parent = parent
    
    def execute(self,command):
        print command
        self.parent._expectedQuery.addActual(command)
        return self.parent.num_returnedRecords
    def fetchall(self):
        return self.parent.returnedRecords

if __name__=="__main__":
    unittest.main(argv=('','-v'))
